@@ -4,6 +4,8 @@ import numpy as np
 from scipy import sparse
 from scipy.sparse.linalg import spsolve_triangular
 import FCOEFF
+import gc
+
 def makeA(data_ord):
     if not isinstance(data_ord, pd.DataFrame): ###必须是data.frame
         print("Please provide data with dataframe type!")
@@ -76,13 +78,21 @@ def makeA(data_ord):
             "data": np.append([-0.5] * (len(nPed["id"])*2),[1.0] *N)
             })
     loc=loc[loc["y"] != N]
-    Tinv = sparse.coo_matrix((loc["data"], (loc["y"], loc["x"])), shape=(N, N), dtype=np.float64).tocsc()
-    di =  sparse.coo_matrix((np.sqrt(1/np.array(dii_c)), (np.array(range(N)), np.array(range(N)))), shape=(N,N), dtype=np.float64).tocsc()
-    X = Tinv.dot(di) ##upper
-
-    #Ip=np.identity(N)
-
+    Tinv = sparse.coo_matrix((loc["data"], (loc["y"], loc["x"])), shape=(N, N), dtype=np.float64).tocsr()
+    print("Tinv")
+    di =  sparse.coo_matrix((np.sqrt(1/np.array(dii_c)), (np.array(range(N)), np.array(range(N)))), shape=(N,N), dtype=np.float64).tocsr()
+    #X = Tinv.dot(di).tocsc() ##upper
+    X = sparse.csr_matrix(Tinv.dot(di),dtype=np.float32)
+    print("X")
+    Ip = sparse.identity(N,dtype='float32', format='dia').toarray() ## 最快的方法 7.7秒3W int8
+    print("ip")
+    tu = spsolve_triangular(X,Ip,False,overwrite_A=True, overwrite_b=True)
+    #tu = sparse.csr_matrix(tu,dtype=np.float32)
+    print("tu")
+    del Ip, X
+    gc.collect()
     #tu = spsolve_triangular(X,Ip,False)
-    tu = sparse.linalg.inv(X)
-    A = (tu.T).dot(tu)
+    #tu = sparse.linalg.inv(X).tocsr()
+    #A = tu.T.dot(tu)
+    A = np.einsum('ij,jk->ik',tu.T,tu)
     return [A, data_ord.iloc[:,0]]
\ No newline at end of file
